'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var postcss = require('postcss');
var postcssrc = require('postcss-load-config');
var deasyncPromise = require('deasync-promise');

// styled component expression regex
var expressionsRegex = /__QUASI_EXPR_(\d+)__/g;

var splitExpressions = function splitExpressions(css) {
  var found = void 0;
  var matches = [];
  while ((found = expressionsRegex.exec(css)) !== null) {
    matches.push(found);
  }

  var _matches$reduce = matches.reduce(function (acc, match) {
    acc.quasiTerms.push(css.substring(acc.prevEnd, match.index));

    var _match = _slicedToArray(match, 2),
        placeholder = _match[0],
        expressionIndex = _match[1];

    acc.expressionTerms.push(expressionIndex);
    acc.prevEnd = match.index + placeholder.length;
    return acc;
  }, { prevEnd: 0, quasiTerms: [], expressionTerms: [] }),
      prevEnd = _matches$reduce.prevEnd,
      quasiTerms = _matches$reduce.quasiTerms,
      expressionTerms = _matches$reduce.expressionTerms;

  quasiTerms.push(css.substring(prevEnd, css.length));
  return { quasiTerms: quasiTerms, expressionTerms: expressionTerms };
};

var buildQuasisAst = function buildQuasisAst(t, terms) {
  return terms.map(function (term, i) {
    return t.templateElement({
      raw: term,
      cooked: term
    }, i === terms.length - 1);
  });
};

var generateExpressionPlaceholder = function generateExpressionPlaceholder(i) {
  return '__QUASI_EXPR_' + i + '__';
};

module.exports = function (_ref) {
  var t = _ref.types;
  return {
    visitor: {
      TaggedTemplateExpression: function TaggedTemplateExpression(path) {
        var _path$node = path.node,
            tag = _path$node.tag,
            _path$node$quasi = _path$node.quasi,
            quasis = _path$node$quasi.quasis,
            expressions = _path$node$quasi.expressions;

        var tmatch = function tmatch() {
          return tag.object && tag.object.name && tag.object.name === 'styled' || tag.property && tag.property.name && tag.property.name === 'extend' || tag.callee && tag.callee.name && tag.callee.name === 'styled' || tag.callee && tag.callee.object && tag.callee.object.object && tag.callee.object.object.name && tag.callee.object.object.name === 'styled' || tag.name && tag.name === 'injectGlobal';
        };

        if (!tmatch()) return;

        // extracts css from template literal
        var css = quasis.reduce(function (acc, _ref2, i) {
          var raw = _ref2.value.raw;

          var expr = expressions[i] ? generateExpressionPlaceholder(i) : '';
          return '' + acc + raw + expr;
        }, '');

        // const processed = postcss(plugins).process(css, options).css

        var _deasyncPromise = deasyncPromise(postcssrc()),
            plugins = _deasyncPromise.plugins,
            options = _deasyncPromise.options;

        var _postcss$process = postcss(plugins).process(css, options);

        css = _postcss$process.css;

        var _splitExpressions = splitExpressions(css),
            quasiTerms = _splitExpressions.quasiTerms,
            expressionTerms = _splitExpressions.expressionTerms;

        var quasisAst = buildQuasisAst(t, quasiTerms);
        var expressionsAst = expressionTerms.map(function (exprIndex) {
          return expressions[exprIndex];
        });

        quasis.splice.apply(quasis, [0, quasis.length].concat(_toConsumableArray(quasisAst)));
        expressions.splice.apply(expressions, [0, expressions.length].concat(_toConsumableArray(expressionsAst)));
      }
    }
  };
};